#!/usr/bin/env bash

#*****************************************************************************
# xtas.sh version 1.1
# ネットワーク流推定手法によるブランドスイッチ確率の算出
# Y.Hamuro
# 2004/10/13
# 2004/10/30 シグナル対応など(1.1)
#*****************************************************************************

#エラーメッセージ
function help {
cat >/dev/stderr <<EOF
--------------------------
xtnflow.sh version 1.1
--------------------------
概要) ネットワーク流量推定手法によるブランド流出入確率の算出
書式) xtnflow.sh -c 顧客項目名 -b ブランド項目名 -q 数量項目名
                 -i 前期ファイル -I 後期ファイル [-o出力ファイル名]
                 [-T ワークディレクトリ名] [-V] [-h]

解説) -b, -q には複数項目を指定できない(単一項目のみ)。
      -q で指定された項目にNULL値があった場合は、その行全てが省かれる。
      -c,-bで指定された項目にNULL値があった場合は、そのまま処理される。

例) xtnflow.sh -c 顧客 -b ブランド -q 数量 -i dat1.xt -I dat2.xt -o rsl.xt
EOF
exit 1
}

#デフォルトでは、コマンドのメッセージはOFF
export mssQuiet=1

#完了メッセージ用にコマンドラインを保存
cmdLine="$0 $*"

#シグナルによる終了
function endByInt {
  rm -f $TD-xx*
  echo "#ERROR# $$" \"$cmdLine\" \""end by signal(ctr^C)"\" >/dev/stderr
  exit 1
}


#パラメータのチェック＆セット
  while getopts ":c:b:q:i:I:o:T:Vh" opt; do
    case $opt in
      c  ) CUST=$OPTARG ;;
      b  ) BRAND=$OPTARG ;;
      q  ) QTTY=$OPTARG ;;
      i  ) input1=$OPTARG ;;
      I  ) input2=$OPTARG ;;
      o  ) output=$OPTARG ;;
      T  ) tmpD=$OPTARG ;;
      V  ) mssQuiet=0 ;;
      h  ) echo 'h'; help ;;
      \? ) echo '?'$OPTARG; help ;;
    esac
  done
  shift $(($OPTIND -1 ))

  if [ "$tmpD" = "" ]; then
     tmpD=/tmp
  fi
  if [ "$input1" = "" ]; then
     echo "#MSG# $$ \"xtas.sh\" \"option -i is mandatory\"" >/dev/stderr
     help; exit 1
  fi
  if [ "$input2" = "" ]; then
     echo "#MSG# $$ \"xtas.sh\" \"option -I is mandatory\"" >/dev/stderr
     help; exit 1
  fi
  if [ "$output" = "" ]; then
     output=/dev/stdout
  fi
  if [ "$CUST" = "" ]; then
     echo "#MSG# $$ \"xtas.sh\" \"option -c is mandatory\"" >/dev/stderr
     help; exit 1
  fi
  if [ "$BRAND" = "" ]; then
     echo "#MSG# $$ \"xtas.sh\" \"option -b is mandatory\"" >/dev/stderr
     help; exit 1
  fi
  if [ "$QTTY" = "" ]; then
     echo "#MSG# $$ \"xtas.sh\" \"option -q is mandatory\"" >/dev/stderr
     help; exit 1
  fi

# ワークファイルのプレフィックスの設定
TD=$tmpD/xxx$$

#トラップ発動
trap endByInt INT QUIT TERM HUP


# ==============================================================================
# 手順１) ブランド0(他店)を求めオリジナルデータに加える(xxt1base,xxt2base)
# ==============================================================================
echo "#MSG# $$ \"Calculating brand 0 on each customer\"" >/dev/stderr

echo "#MSG# $$ \" ... cutting out necessary fields and summing up\"" >/dev/stderr
xtcut -f "$CUST,$BRAND:##b1##,$QTTY:##q1##" -i $input1 |
xtdelnul -f "##q1##" |
xtagg -k "$CUST,##b1##" -f "##q1##" -c sum -o $TD-xxt1

xtcut -f "$CUST,$BRAND:##b2##,$QTTY:##q2##" -i $input2 |
xtdelnul -f "##q2##" |
xtagg -k "$CUST,##b2##" -f "##q2##" -c sum -o $TD-xxt2

echo "#MSG# $$ \" ... calculating a difference of quantity between T1 and T2\"" >/dev/stderr
xtcut -f $CUST,##q2## -i $TD-xxt2 |
xtagg -k $CUST -f "##q2##" -c sum -o $TD-xxa

xtcut -f $CUST,##q1## -i $TD-xxt1 |
xtagg -k $CUST -f "##q1##" -c sum |
xtjoin -k $CUST -m $TD-xxa -f "##q2##" -nN |
xtnulto -f "##q1##,##q2##" -v 0 |
xtsel -c '$##q2## >= $##q1##' -u $TD-xxq1g |
xtsel -c '$##q2## <> $##q1##' -o $TD-xxq2g

echo "#MSG# $$ \" ... generating Brand0 files\"" >/dev/stderr
xtcal -c '$##q2##-$##q1##' -a "##diff##" -i $TD-xxq2g |
xtsetchr -v "#BRAND0#" -a "##b1##" |
xtcut -f "$CUST,##b1##,##diff##:##q1##" -o $TD-xxb0t1

xtcal -c '$##q1##-$##q2##' -a "##diff##" -i $TD-xxq1g |
xtsetchr -v "#BRAND0#" -a "##b2##" |
xtcut -f "$CUST,##b2##,##diff##:##q2##" -o $TD-xxb0t2

echo "#MSG# $$ \" ... concatinate Brand0 files and original files\"" >/dev/stderr
xtcat -i $TD-xxt1,$TD-xxb0t1 -o $TD-xxt1base
xtcat -i $TD-xxt2,$TD-xxb0t2 -o $TD-xxt2base

# ==============================================================================
# 手順2) 顧客別合計を求め、t1*t2(自然結合)表に結合する
# ==============================================================================
echo "#MSG# $$ \"Calculating total quantity on each customer\"" >/dev/stderr
xtcut -f "$CUST,##q1##" -i $TD-xxt1base |
xtagg -k $CUST -f "##q1##:##ctotal##" -c sum -o $TD-xxctotal

xtcut -f "##q1##" -i $TD-xxt1base |
xtagg -f "##q1##:##total##" -c sum -o $TD-xxtotal

echo "#MSG# $$ \"Calculating total quantity on each brand\"" >/dev/stderr
xtcut -f "##b1##,##q1##" -i $TD-xxt1base |
xtagg -k "##b1##" -f "##q1##:##q1Total##" -c sum -o $TD-xxq1Total

xtcut -f "##b2##,##q2##" -i $TD-xxt2base |
xtagg -k "##b2##" -f "##q2##:##q2Total##" -c sum -o $TD-xxq2Total

echo "#MSG# $$ \"Calculating network flow estimated\"" >/dev/stderr
xtcut   -f "$CUST,##b1##,##q1##" -i $TD-xxt1base |
xtnjoin -k $CUST -m $TD-xxt2base -f "##b2##,##q2##" |
xtjoin  -k $CUST -m $TD-xxctotal -f "##ctotal##" |
xtcal -c '$##q1##*$##q2##/$##ctotal##' -a "##flow##" |
xtcut -f "##b1##,##b2##,##flow##" |
xtagg -k "##b1##,##b2##" -f "##flow##" -c sum |
xtjoin -k "##b1##" -m $TD-xxq1Total -f "##q1Total##" |
xtjoin -k "##b2##" -m $TD-xxq2Total -f "##q2Total##" |
xtproduct -m $TD-xxtotal -f "##total##" |
xtcal -c '$##flow##/$##q1Total##' -a "##op##" |
xtcal -c '$##flow##/$##q2Total##' -a "##ip##" |
xtcal -c '$##flow##/$##total##' -a "##support##" |
xtcut -f "##b1##:B1,##b2##:B2,##flow##:Flow,##q1Total##:Q1,##q2Total##:Q2,##total##:Total,##op##:flowOutProb,##ip##:flowInProb,##support##:support" -o $output
echo "#MSG# $$ \"Done\"" >/dev/stderr

rm $TD-xx*

